import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

mixin CountableController on GetxController {
  RxSet<String> _tasks = RxSet();
  RxBool isProcessing = RxBool(false);

  @override
  void onReady() {
    super.onReady();
    autoDispose(
      _tasks.listen(
        (data) {
          if (isProcessing.value != data.isNotEmpty) {
            isProcessing.toggle();
          }
        },
      ),
    );
  }

  List<StreamSubscription> _streams = [];

  @override
  void dispose() {
    _streams
      ..forEach((element) => element.cancel())
      ..clear();
    super.dispose();
  }

  void autoDispose(StreamSubscription subscription) =>
      _streams.add(subscription);

  void add(
    String uuid, {
    Duration autoComplete = const Duration(seconds: 5),
  }) {
    postFrameFuture(() {
      _tasks.add(uuid);
      Future.delayed(autoComplete).then((_) => complete(uuid));
    });
  }

  void complete(String uuid) => _tasks.remove(uuid);
}
